# -*- tcl -*-
#		Pool 2.2, as of February 20, 1999
#		Pool_Clock @clock:mFullVersion@
#
# CVS: $Id: calendar.cls,v 1.7 1998/06/02 20:05:09 aku Exp $
#
# @c Calendar widget to choose a date.
# @s Calendar widget
# @i calendar, choosing a date, date choice
# @see <c yearBrowser>, <c monthList>, <c dayBrowser>, <c calendarTix>
# -------------------------------------------

class calendar : valueManager {

    init {
	package require Pool_GuiBase
    }

    # @o heading: Color to mark header row.
    # @o sunday:  Color to mark sundays.
    # @o weekno:  Color to mark week numbers

    option heading -d black -a TrackHeading
    option sunday  -d red   -a TrackSunday
    option weekno  -d black -a TrackWeekno

    # @o dayrelief: Relief of button associated to currently chosen day

    option dayrelief -d sunken -a TrackDayRelief -a ::pool::ui::relief


    # Current opt(-value), broken down into parts.
    #
    # @v year:  Currently chosen year
    # @v month: Currently chosen month
    # @v day:   Currently chosen day in the month

    var year
    var month
    var day


    method calendar {} {
	# @c Initializes the value to the current date if not set already by
	# @c the user.

	set now [::pool::date::now]

	if {[string compare $opt(-value) ""] == 0} {
	    set opt(-value) $now
	}

	# Compute derived information for the first time.

	::pool::date::split $now year month day
	return
    }



    method createSubwidgets {} {
	# @c Called by the framework to generate the subwidgets and their
	# @c layout.

	yearBrowser $this.year             \
		-value              $year  \
		-highlightthickness 0
		
	monthList $this.month                 \
		-value              $month    \
		-highlightthickness 0         \
		-relief             sunken    \
		-bd                 2         \
		-onwrap             $this.year

	dayBrowser $this.day                       \
		-value              $day           \
		-highlightthickness 0              \
		-heading            $opt(-heading) \
		-sunday             $opt(-sunday)  \
		-weekno             $opt(-weekno)  \
		-onwrap             $this.month

	label $this.date                      \
		-bd                 3         \
		-relief             groove    \
		-text               {}        \
		-highlightthickness 0

	$this.year  configure -command [list ${this}::TrackYear]
	$this.month configure -command [list ${this}::TrackMonth]
	$this.day   configure -command [list ${this}::TrackDay]

	grid $this.year   -column 0 -row 0 -sticky swen -in $this
	grid $this.month  -column 0 -row 1 -sticky swen -in $this
	grid $this.day    -column 1 -row 1 -sticky swen -in $this
	grid $this.date   -column 1 -row 0 -sticky swen -in $this

	grid columnconfigure $this 0 -weight 1
	grid columnconfigure $this 1 -weight 1
	grid rowconfigure    $this 0 -weight 1
	grid rowconfigure    $this 1 -weight 1
	return
    }



    method setBindings {} {
	# @c Adds various keyboard accelerators to the calendar components.

	$this.day   setBindingsTo $this
	$this.day   setBindingsTo $this.year
	$this.day   setBindingsTo $this.month
	$this.day   setBindingsTo $this.date

	$this.month setBindingsTo $this
	$this.month setBindingsTo $this.day
	$this.month setBindingsTo $this.year
	$this.month setBindingsTo $this.date

	$this.year  setBindingsTo $this
	$this.year  setBindingsTo $this.day
	$this.year  setBindingsTo $this.month
	$this.year  setBindingsTo $this.date
	return
    }



    method setBindingsTo {w} {
	# @c Adds various keyboard accelerators to the given widget.
	# @a w: The widget the accelerators are added to.

	$this.day   setBindingsTo $w
	$this.month setBindingsTo $w
	$this.year  setBindingsTo $w
	return
    }



    method TrackHeading {o oldValue} {
	# @c Executed whenever the color of the heading is changed.
	#
	# @a o: The name of the changed option, always '-heading'.
	# @a oldValue: The old value of the option.

	$this.day configure -heading $opt(-heading)
	return
    }



    method TrackSunday {o oldValue} {
	# @c Executed whenever the color of the sunday column
	# @c requires a refresh.
	#
	# @a o: The name of the changed option, always '-sunday'.
	# @a oldValue: The old value of the option.

	$this.day configure -sunday $opt(-sunday)
	return
    }



    method TrackWeekno {o oldValue} {
	# @c Executed whenever the color of the weeknumber
	# @c column requires a refresh.
	#
	# @a o: The name of the changed option, always '-weekno'.
	# @a oldValue: The old value of the option.

	$this.day configure -weekno $opt(-weekno)
	return
    }



    method TrackDayrelief {o oldValue} {
	# @c Executed whenever the relief of the buttons is set.
	#
	# @a o: The name of the changed option, always '-dayrelief'.
	# @a oldValue: The old value of the option.

	$this.day configure -dayrelief $opt(-dayrelief)
	return
    }



    method updateDisplay {} {
	# @c Method required by the superclass <c valueManager>
	# @c to propagate changes to the value into the display.

	::pool::date::split $opt(-value) year month day

	$this.year   configure -value $year
	$this.month  configure -value $month
	$this.day    configure -value $day \
		-month [::pool::date::2month $opt(-value)]
	$this.date   configure -text  \
		[clock format [clock scan $opt(-value)] -format {%d. %B %Y}]

	return
    }



    method TrackYear  {y} {
	# @c Callback for subordinate year widget.
	# @c Executed every time its value is changed.
	# @a y: The new value of the chosen year.

	set year $y

	$this.day configure -month [::pool::date::joinMonth $y $month]
	configure           -value [::pool::date::join      $y $month $day]
	return
    }



    method TrackMonth {m} {
	# @c Callback for subordinate month widget.
	# @c Executed every time its value is changed.
	# @a m: The new value of the chosen month.

	set month $m
	$this.day configure -month [::pool::date::joinMonth $year $m]
	configure           -value [::pool::date::join      $year $m $day]
	return
    }



    method TrackDay {d} {
	# @c Callback for subordinate day widget.
	# @c Executed every time its value is set (not necessarily changed).
	# @n Does nothing, if no change took place (avoids infinite recursion).
	# @a d: The (possibly) new value of the chosen day.

	configure -value [::pool::date::join $year $month $d]
	return
    }
}

